//+----------------------------------------------------------------------------+
//|                               mp6140 GBPUSD Daily Hedge EA for MetaTrader4 |
//|                                                               version 3.21 |
//|                                                  original idea from mp6140 |
//|                                                     EA conversion by azmel |
//+----------------------------------------------------------------------------+
//| VERSION HISTORY                                                            |
//| 1   Initial version. Settings for three AccountTypes. New trades won't     |
//|     open until previous trades closes.                                     |
//| 2   Fixed SELL bug                                                         |
//| 3   Added Money Management and Take Profit. New hedge opens regardless of  |
//|     previous hedge closed or not.                                          |
//| 3.1 Hedge can be opened by the minute instead of by the hour.              | 
//| 3.2 Added UseBreakEven feature and customisable buy/sell comments.         |
//|     TimeToTrade entry is now entered with a four-character text.           |
//| 3.21 AccountType now is automatic. MoneyManagement is re-written.          |
//+----------------------------------------------------------------------------+
//| INSTRUCTIONS                                                               |
//| To be used on GBPUSD pair only with any timeframe. EA will place a hegde   |
//| (a BUY and a SELL simultaneously) at the specified time. Break Even        |
//| feature is introduced whereby if one of the trade hit its TakeProfit's     |
//| value, the other trade will move to Break Even, to minimise risk.          |
//+----------------------------------------------------------------------------+
//| SETTINGS                                                                   |
//| AccountType=0 for Standard (1 Lot) Accounts                                |
//| AccountType=1 for Mini (0.1 Lot) Accounts                                  |
//| AccountType=2 for Micro (0.01 Lot) Accounts                                |
//| AccountType=3 for InterbankFX Nano (0.01 Lot) Accounts                     |
//+----------------------------------------------------------------------------+

#include <stdlib.mqh>

//+----------------------------------------------------------------------------+
//| External Variables                                                         |
//+----------------------------------------------------------------------------+

extern int    Magic             = 54281;
extern string TimeToTrade       = "0115";
extern double Lots              = 0.01;
extern int    TakeProfit        = 5;
extern int    StopLoss          = 0;
extern bool   MoneyManagement   = false;
extern int    Risk              = 10;
extern bool   UseBreakEven      = false;
extern string BuyComment        = "mp6140-buy";
extern string SellComment       = "mp6140-sell";

//+----------------------------------------------------------------------------+
//| Internal Variables                                                         |
//+----------------------------------------------------------------------------+

double   MaxLots;
double   DecimalCorrection;
int      DecimalPlaces;
double   PricePerPip;

int      HourToTrade;
int      MinuteToTrade;

bool     OKToTrade;
int      Status = 1;
int      UseStopLoss;
int      BuyFlag;
int      SellFlag;
int      SecureBuyFlag;
int      SecureSellFlag;

int      ticket1;
int      ticket2;
int      ticket3;
double   TP1;
double   TP2;
double   TP3;
double   L3;
int      err;
int      slippage=5;

// Completed Phases (for reference only):
// 0 - No trade allowed.
// 1 - Standby and waiting.
// 2 - Buy and Sell Placed.
// 3 - Buy or Sell Moved to Break Even.
// 4 - (Reserved for future development).

//+----------------------------------------------------------------------------+
//| Main code                                                                  |
//+----------------------------------------------------------------------------+

int start()
{
   if (MoneyManagement)
   {
      if (Risk<1 || Risk>50)
      {
         Comment("Invalid Risk Value.");
      }
      else
      {
         Lots=MathFloor((AccountFreeMargin()*AccountLeverage()*Risk*Point*100)/(Ask*MarketInfo(Symbol(),MODE_LOTSIZE)*MarketInfo(Symbol(),MODE_MINLOT)))*MarketInfo(Symbol(),MODE_MINLOT);
      }
   }

   if (StringLen(TimeToTrade)==4)
   {
      HourToTrade=StrToInteger(StringSubstr(TimeToTrade,0,2));
      MinuteToTrade=StrToInteger(StringSubstr(TimeToTrade,2,2));
      if (HourToTrade<0 || HourToTrade>23 || MinuteToTrade<0 || MinuteToTrade>59)
      {
         Comment("Hedge Time Error.");
         OKToTrade=false;
      }
   }
   else
   {
      Comment("Hedge Time Error.");
      OKToTrade=false;
   }
   
   if (StopLoss==0)
   {
      UseStopLoss=0;
   }
   else
   {
      UseStopLoss=1;
   }  
   
   if (OKToTrade && Status==1 && Hour()==HourToTrade && Minute()==MinuteToTrade)
   {
      if (BuyFlag==0)
      {
         ticket1=OrderSend(Symbol(),OP_BUY,Lots,Ask,slippage
            ,(Ask-StopLoss*Point)*UseStopLoss,Ask+TakeProfit*Point,BuyComment
            ,Magic,CLR_NONE);
         if (ticket1 == -1)
         {
            Print("BUY ERROR: ",ErrorDescription(GetLastError())," at ", Hour()
               ,":",Minute());
         }
         else
         {
            OrderSelect(ticket1,SELECT_BY_TICKET);
            TP1=OrderTakeProfit();         
            BuyFlag=1;
         }
      }


      if (SellFlag==0)
      {
         ticket2=OrderSend(Symbol(),OP_SELL,Lots,Bid,slippage
            ,(Bid+StopLoss*Point)*UseStopLoss,Bid-TakeProfit*Point,SellComment
            ,Magic,CLR_NONE);
         if (ticket2 == -1)
         {
            Print("SELL ERROR: ",ErrorDescription(GetLastError())," at ", Hour()
               ,":",Minute());
         }
         else
         {
            OrderSelect(ticket2,SELECT_BY_TICKET);
            TP2=OrderTakeProfit();         
            SellFlag=1;
         }
      }

      if (BuyFlag && SellFlag)
      {
         Status=2;
      }
   }

   if (Status==2 && UseBreakEven)
   {
      if (Bid >= TP1 && SecureSellFlag==0)
      {
         OrderSelect(ticket2,SELECT_BY_TICKET);
         TP3=OrderOpenPrice();
         L3=OrderLots();
         if (OrderModify(ticket2,OrderOpenPrice(),OrderStopLoss(),TP3,0,CLR_NONE))
         {
            SecureSellFlag=1;
         }
         else
         {
            Print("SECURE SELL ERROR: ",ErrorDescription(GetLastError())," at "
               , Hour(),":",Minute());            
         }
      }

      if (Ask <= TP2 && SecureBuyFlag==0)
      {
         OrderSelect(ticket1,SELECT_BY_TICKET);
         TP3=OrderOpenPrice();
         L3=OrderLots();
         if (OrderModify(ticket1,OrderOpenPrice(),OrderStopLoss(),TP3,0,CLR_NONE))
         {
            SecureBuyFlag=1;
         }
         else
         {
            Print("SECURE BUY ERROR: ",ErrorDescription(GetLastError())," at "
               , Hour(),":",Minute());            
         }
      }
         
      if (SecureSellFlag || SecureBuyFlag)
      {
         Status=3;
      }
   }

   if (Status==2 && UseBreakEven==false)   
   {
      Status=3;
   }

   //+-------------------------------------------------------------------------+
   //| Status 3 is reserved for future development                             |
   //+-------------------------------------------------------------------------+

   if (Status==3)
   {
      Status=4;
      BuyFlag=0;
      SellFlag=0;
      SecureBuyFlag=0;
      SecureSellFlag=0;
   }

   if (Status==4 && Hour() != HourToTrade)
   {
      Status=1;
   }


   if (OKToTrade)
   {
      if (MinuteToTrade<10)
      {
         Comment("Lots = ",Lots,"\nHedge time = ",HourToTrade,":0",MinuteToTrade);
      }
      else
      {
         Comment("Lots = ",Lots,"\nHedge time = ",HourToTrade,":",MinuteToTrade);
      }
   }
   

   return(0);
}